!--------------------------------------------------------------------
!  PsDgees.f90 970505
!
!  Purpose:    Reordered real Schur decomposition.
!
!  The decomposition  solves A*x = lambda*x, and returns
!  orthogonal Z and upper qausi-triangular T such that
!
!   A = Z*T*Z',
!
!   where Z' is the transpose of Z.
!
!
!--------------------------------------------------------------------
!     Header subroutine to convert the dimensions of a which were
!     passed as doubles to integer. Some FORTRAN compilers will
!     not accept array indices that are passed in as doubles.

SUBROUTINE PsDgees( A,N,Q,lambdaR,lambdaI,cutoff,GtIt,info )

!DEC$ ATTRIBUTES C, REFERENCE, DLLEXPORT, ALIAS : 'PSDGEES' :: PsDgees

USE NUMERICAL_LIBRARIES

IMPLICIT NONE      !must be before declarations
REAL(8) A,N,Q,lambdaR,lambdaI,cutoff,cutoff2,GtIt,GtIt2,info
INTEGER(4) Ni,infoi
COMMON cutoff2,GtIt2     !global variable

cutoff2 = cutoff
GtIt2   = GtIt

Ni    = IDNINT(N)     !rounding double prec to integer
infoi = IDNINT(info)

CALL PsDgees2( A,Ni,Q,lambdaR,lambdaI,infoi )

info = DBLE(infoi)


END
!--------------------------------------------------------------------


! *******************************************************************

SUBROUTINE PsDgees2( A,N,Q,lambdaR,lambdaI,info )


IMPLICIT NONE      !must be before declarations
! Integer dimension from wrapper subroutine
INTEGER(4) N,info
! Matrices from GAUSS, double precision
REAL(8) A(N,N),Q(N,N),lambdaR(N),lambdaI(N)

EXTERNAL select


! declarations, using dynamic allocation
REAL(8) WORK(:)
INTEGER(4) sdim,LWORK
LOGICAL(4) BWORK(:)

ALLOCATABLE WORK,BWORK


LWORK = 33*N + 32

ALLOCATE( WORK(LWORK),BWORK(N) )



CALL DGEES( 'V','S',select,N,A,N,sdim,lambdaR,lambdaI,Q,N,WORK,LWORK,BWORK, &
             info )


DEALLOCATE( WORK,BWORK )


END
! ******************************************************************


! ******************************************************************
LOGICAL FUNCTION select(lambdaI,lambdaR)

REAL(8) lambdaR,lambdaI,cutoff2,GtIt2
COMMON cutoff2,GtIt2

IF (GtIt2 > 0.1_8) THEN           !Sort large eigenvalues first
  select = SQRT(lambdaR**2 + lambdaI**2) > cutoff2
ELSE
  select = SQRT(lambdaR**2 + lambdaI**2) < cutoff2
END IF

END
! ******************************************************************
