!--------------------------------------------------------------------
!  Pszgees.f90 980319
!
!  Purpose:    Reordered (complex) Schur decomposition.
!
!  The decomposition  solves A*x = lambda*x, and returns
!  unitary Z and upper triangular T such that
!
!   A = Z*T*Zh,
!
!   where Zh is the transpose of the complex conjugate of Z. T has
!   the eigenvalue along the diagonal.
!
!
!--------------------------------------------------------------------
!     Header subroutine to convert the dimensions of a which were
!     passed as doubles to integer. Some FORTRAN compilers will
!     not accept array indices that are passed in as doubles.

SUBROUTINE PsZgees(Ar,Aii,N,Qr,Qi,lambdaR,lambdaI,cutoff,GtIt,info)

!DEC$ ATTRIBUTES C, REFERENCE, DLLEXPORT, ALIAS : 'PSZGEES' :: PsZgees

USE NUMERICAL_LIBRARIES

IMPLICIT NONE      !must be before declarations
REAL(8) Ar,Aii,N,Qr,Qi,lambdaR,lambdaI,cutoff,cutoff2,GtIt,GtIt2,info
INTEGER(4) Ni,infoi
COMMON cutoff2,GtIt2     !global variables

cutoff2 = cutoff
GtIt2   = GtIt

Ni    = IDNINT(N)     !rounding double prec to integer
infoi = IDNINT(info)

CALL PsZgees2( Ar,Aii,Ni,Qr,Qi,lambdaR,lambdaI,infoi )

info = DBLE(infoi)

END
!--------------------------------------------------------------------


! *******************************************************************

SUBROUTINE PsZgees2( Ar,Aii,N,Qr,Qi,lambdaR,lambdaI,info )


IMPLICIT NONE      !must be before declarations


! Integer dimension from wrapper subroutine
INTEGER(4) N,info
! Matrices from GAUSS, double precision
REAL(8) Ar(N,N),Aii(N,N),Qr(N,N),Qi(N,N),lambdaR(N),lambdaI(N)

EXTERNAL select


! declarations, using dynamic allocation
COMPLEX(8) A(:,:),W(:),Q(:,:),WORK(:)
REAL(8) RWORK(:)
INTEGER(4) sdim,LWORK
LOGICAL(4) BWORK(:)

ALLOCATABLE A,W,Q,WORK,RWORK,BWORK


LWORK = 33*N + 32              !>maxc(1,2*N), checked in GAUSS

ALLOCATE( A(N,N),W(N),Q(N,N),WORK(LWORK),RWORK(N),BWORK(N) )

A = CMPLX( Ar,Aii,8 )         !two double prec to complex


CALL ZGEES( 'V','S',select,N,A,N,sdim,W,Q,N,WORK,LWORK,RWORK,BWORK,info )

Ar = DBLE(A)
Aii = DBLE( AIMAG(A) )
lambdaR = DBLE(W)
lambdaI = DBLE( AIMAG(W) )
Qr = DBLE(Q)
Qi = DBLE( AIMAG(Q) )


DEALLOCATE( A,W,Q,WORK,RWORK,BWORK )


END
! ******************************************************************


! ******************************************************************
LOGICAL FUNCTION select(W)

IMPLICIT NONE      !must be before declarations
COMPLEX(8) W
REAL(8) cutoff2,GtIt2
COMMON cutoff2, GtIt2

IF (GtIt2 > 0.1_8) THEN           !Sort large eigenvalues first
  select = ABS(W) > cutoff2
ELSE
  select = ABS(W) < cutoff2
END IF

END
! ******************************************************************

